==> frostiful-2.2.6%2B1.21.1-sources.jar: com/github/thedeathlycow/frostiful/server/world/IcicleWeatherGenerator.java <==
package com.github.thedeathlycow.frostiful.server.world;


import com.github.thedeathlycow.frostiful.Frostiful;
import com.github.thedeathlycow.frostiful.block.IcicleBlock;
import com.github.thedeathlycow.frostiful.config.group.IcicleConfigGroup;
import com.github.thedeathlycow.frostiful.registry.FBlocks;
import java.util.function.Predicate;
import net.minecraft.class_1923;
import net.minecraft.class_1944;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2680;
import net.minecraft.class_2818;
import net.minecraft.class_2902;
import net.minecraft.class_3218;
import net.minecraft.class_5819;

public final class IcicleWeatherGenerator {

    public static void tickIciclesForChunk(class_3218 world, class_2818 chunk, int randomTickSpeed) {

        final class_5819 random = world.field_9229;

        IcicleConfigGroup icicleConfig = Frostiful.getConfig().icicleConfig;
        if (!icicleConfig.iciclesFormInWeather()) {
            return;
        }

        if (!world.method_8419()) {
            return;
        }

        // slow down icicles appearing in world
        if (random.method_43048(16 * 5) != 0) {
            return;
        }

        final class_1923 chunkPos = chunk.method_12004();
        final class_2338 startPos = world.method_8598(
                class_2902.class_2903.field_13202,
                world.method_8536(chunkPos.method_8326(), 0, chunkPos.method_8328(), 15)
        );

        if (world.method_23753(startPos).comp_349().method_39927(startPos)) {
            return;
        }

        final class_2680 downwardIcicle = FBlocks.ICICLE.method_9564()
                .method_11657(IcicleBlock.VERTICAL_DIRECTION, class_2350.field_11033);

        Predicate<class_2338> validCondition = (testPos) -> {
            class_2680 at = world.method_8320(testPos);

            // testing for sky light helps increase the chance that icicles will only ever form outside
            return at.method_26215()
                    && world.method_8314(class_1944.field_9284, testPos) >= icicleConfig.getMinSkylightLevelToForm()
                    && downwardIcicle.method_26184(world, testPos);
        };

        class_2338.class_2339 placePos = startPos.method_25503();
        for (int i = 0; i < 5; i++) {

            if (validCondition.test(placePos)) {
                // only place if can place and light is not blocking it
                if (world.method_8314(class_1944.field_9282, placePos) < icicleConfig.getMaxLightLevelToForm()) {
                    world.method_8652(placePos, downwardIcicle, class_2248.field_31036);
                }

                // if can place but there is light blocking - stop looking
                return;
            }
            placePos.method_10098(class_2350.field_11033);
        }

    }

    private IcicleWeatherGenerator() {

    }

}

==> frostiful-2.2.9%2B1.21.9-sources.jar: com/github/thedeathlycow/frostiful/server/world/IcicleWeatherGenerator.java <==
package com.github.thedeathlycow.frostiful.server.world;


import com.github.thedeathlycow.frostiful.Frostiful;
import com.github.thedeathlycow.frostiful.block.IcicleBlock;
import com.github.thedeathlycow.frostiful.config.group.IcicleConfigGroup;
import com.github.thedeathlycow.frostiful.registry.FBlocks;
import java.util.function.Predicate;
import net.minecraft.class_1923;
import net.minecraft.class_1944;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2680;
import net.minecraft.class_2818;
import net.minecraft.class_2902;
import net.minecraft.class_3218;
import net.minecraft.class_5819;

public final class IcicleWeatherGenerator {

    public static void tickIciclesForChunk(class_3218 world, class_2818 chunk, int randomTickSpeed) {

        final class_5819 random = world.field_9229;

        IcicleConfigGroup icicleConfig = Frostiful.getConfig().icicleConfig;
        if (!icicleConfig.iciclesFormInWeather()) {
            return;
        }

        if (!world.method_8419()) {
            return;
        }

        // slow down icicles appearing in world
        if (random.method_43048(16 * 5) != 0) {
            return;
        }

        final class_1923 chunkPos = chunk.method_12004();
        final class_2338 startPos = world.method_8598(
                class_2902.class_2903.field_13202,
                world.method_8536(chunkPos.method_8326(), 0, chunkPos.method_8328(), 15)
        );

        if (world.method_23753(startPos).comp_349().method_39927(startPos, world.method_8615())) {
            return;
        }

        final class_2680 downwardIcicle = FBlocks.ICICLE.method_9564()
                .method_11657(IcicleBlock.VERTICAL_DIRECTION, class_2350.field_11033);

        Predicate<class_2338> validCondition = (testPos) -> {
            class_2680 at = world.method_8320(testPos);

            // testing for sky light helps increase the chance that icicles will only ever form outside
            return at.method_26215()
                    && world.method_8314(class_1944.field_9284, testPos) >= icicleConfig.getMinSkylightLevelToForm()
                    && downwardIcicle.method_26184(world, testPos);
        };

        class_2338.class_2339 placePos = startPos.method_25503();
        for (int i = 0; i < 5; i++) {

            if (validCondition.test(placePos)) {
                // only place if can place and light is not blocking it
                if (world.method_8314(class_1944.field_9282, placePos) < icicleConfig.getMaxLightLevelToForm()) {
                    world.method_8652(placePos, downwardIcicle, class_2248.field_31036);
                }

                // if can place but there is light blocking - stop looking
                return;
            }
            placePos.method_10098(class_2350.field_11033);
        }

    }

    private IcicleWeatherGenerator() {

    }

}
